/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.util;

import java.awt.Component;
import java.awt.Desktop;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.FlavorListener;
import java.awt.datatransfer.StringSelection;
import java.io.IOException;
import java.nio.file.Path;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableModel;

public final class GuiUtil {
	public static final int K_BYTE=1024,M_BYTE=1048576,G_BYTE=1073741824;
	private static DecimalFormat dfBytes=new DecimalFormat("0 B");
	private static DecimalFormat dfKBytes=new DecimalFormat("0.0 KB");
	private static DecimalFormat dfMBytes=new DecimalFormat("0.0 MB");
	private static DecimalFormat dfGBytes=new DecimalFormat("0.0 GB");
	private static Calendar cal=GregorianCalendar.getInstance();
	private static DateFormat df=DateFormat.getDateInstance();
	private static DateFormat tf=DateFormat.getTimeInstance();
	public static final Border DEFAULT_EMPTY_BORDER=BorderFactory.createEmptyBorder(10, 10, 10, 10);

	private static Clipboard cb;
	static {
		cb=Toolkit.getDefaultToolkit().getSystemClipboard();
	}
	
	public static String getClipboardString() {
		try {
			if (cb.isDataFlavorAvailable(DataFlavor.stringFlavor))
				return cb.getData(DataFlavor.stringFlavor).toString();
			else 
				return null;
			
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		
	}
	
	public static void addClipboardListener(FlavorListener l) {
		cb.addFlavorListener(l);
	}
	
	public static void removeClipboardListener(FlavorListener l) {
		cb.removeFlavorListener(l);
	}
	
	public static void setClipboardString(String s) {
		StringSelection data=new StringSelection(s);
		cb.setContents(data, null);
	}
	

	
	
	public static void setClipboardString(Collection<String> list) {
		if (list.size()>0) {
			StringBuilder sb=new StringBuilder();
			
			Iterator<String> it=list.iterator();
			while (true) {
				sb.append(it.next());
				
				if (it.hasNext()) {
					sb.append('\n');
				} else {
					break;
				}
			}
			setClipboardString(sb.toString());
			
		}
	}
	
	public static String formatSpeed(double speed) {
		return formatBytes(speed) +"/s";
	}

	public static String formatBytes(double bytes) {
		if (bytes < K_BYTE) {
			if (bytes>=0) { 
				return dfBytes.format(bytes);
			} else {
				return dfBytes.format(0L);
			}
		} else if (bytes < M_BYTE) {
			return dfKBytes.format(bytes/K_BYTE);
		} else if (bytes < G_BYTE) {
			return dfMBytes.format(bytes/M_BYTE);
		} else if (bytes >= G_BYTE) {
			return dfGBytes.format(bytes/G_BYTE);
		} else {
			return dfBytes.format(0L);
		}
			
	}
	
	public static String formatETA(long avgSpeed,long filesize,long bytesFinished) {
		if (avgSpeed<=0) return "\u221E";
		long seconds=(filesize-bytesFinished)/avgSpeed;
		if (seconds<0) {
			return "\u221E";
		} else if (seconds < 60) {
			return seconds + " s";
		} else if (seconds < 3600) {
			long mins =  seconds / 60;
			long restSecs = seconds % 60;
			if (restSecs > 0) {
				String formattedSecs = "";
				if (restSecs < 10) {
					formattedSecs = "0";
				}
				formattedSecs += restSecs;
				return mins + ":" + formattedSecs + " min";
			} else {
				return mins + " min";
			}
		} else {
			long hours = seconds / 3600;
			long mins = (seconds % 3600) / 60;
			String formattedMins = "";
			if (mins < 10) {
				formattedMins = "0";
			}
			formattedMins += mins;
			return hours + ":" + formattedMins + " h";
		}
	}
	
	public static String formatPercent(long bytes,long size) {
		if (size<=0) {
			return "0 %";
		} else {
			return bytes*100/size+ " %";
		}
	}
	
	public static int getXCenterToComponent(Component relativeToComponent,int width) {
		return (relativeToComponent.getWidth()-width)/2+relativeToComponent.getX();
	}
	public static int getYCenterToComponent(Component relativeToComponent,int height) {
		return (relativeToComponent.getHeight()-height)/2+relativeToComponent.getY();
	}

	public static String formatDate(long millis) {
		cal.setTimeInMillis(millis);
		return df.format(cal.getTime());
	}
	
	public static String formatTime(long millis) {
		cal.setTimeInMillis(millis);
		return tf.format(cal.getTime());
	}
	
	public static String formatDateTime(long millis) {
		cal.setTimeInMillis(millis);
		return df.format(cal.getTime()) +" "+tf.format(cal.getTime());
	}
	
	public static void openBrowser(Path path) {
		try {
			Desktop.getDesktop().browse(path.toUri());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void setColumnPositions(JTable table, int[] colPos) {
		if (colPos != null) {
			for (int i = 0; i < colPos.length; i++) {

				int p = colPos[i];

				int k = table.getColumnModel().getColumnIndex(
						table.getModel().getColumnName(p));
				if (k != i) {
					table.getColumnModel().moveColumn(k, i);
				}

			}
		}
	}
	
	public static int[] getColumnPositions(JTable table) {
		int[] columnPos = new int[table.getColumnCount()];

		for (int i = 0; i < table.getColumnCount(); i++) {
			columnPos[i] = ((DefaultTableModel) table.getModel()).findColumn(table
					.getColumnModel().getColumn(i).getHeaderValue().toString());
		}
		return columnPos;
	}
}
